<?php
/********************************************************************
* mini-pub - Mini PHP Publisher
* -----------------------------
*
* Copyright (C) 2002 Fabio A Mazzarino
* Released under the terms of GNU General Public License version 2
* See COPYING file for more information
*
* File: classDataParser.inc.php
* Descr.: xml parser for mini-pub .data files
*
* Authors:
*   Fabio A Mazzarino <mazza at altern.org>
*
* Changes
*   May/24/2002 - mazza - Initial version
*   Jun/13/2002 - mazza - New parsing technic, using xml struct
*   Jan/24/2003 - mazza - data content marks now uses &gt; and &lt;
*
* ToDo:
*
*******************************************************************/

if(!defined("__CLASS_DATA_PARSER_INC_PHP__")){
define("__CLASS_DATA_PARSER_INC_PHP__", "classDataParser.inc.php");


// retrieving include dir
$aIncluded = get_included_files();
while(list($id, $sFile) = each($aIncluded)){
  $iPos = strpos($sFile, "classDataParser.inc.php");
  if($iPos){             // this is the right include
    $sIncludePath = dirname($sFile);
    break;
  }
}

class dataParser{

// parser
var $oParser;

// .data 
var $sDataName;
var $aEntries;

// internal
var $iCurrentId;

// parser
var $aValues;
var $aTags;
var $ctValues;

// error handling
var $sError;
var $iErrorLine;
// warning handling
var $aWarnings;
var $aWarningsLine;
var $ctWarnings;


/* Descr.: default constructor to data parser class
 * Params.:
 * Return:
 *   none
 * Notes:
 * ToDo:
**/
function dataParser(){
  // setting error variables to no error
  $this->sError = null;
  $this->iErrorLine = 0;

  $this->aWarnings = array();
  $this->aWarningsLins = array();
  $this->ctWarnings = 0;

  // multiple templ support
  $this->aEntries = array();

  // clear old data
  unset($this->sDataName);
  unset($this->iCurrentId);
  unset($this->oParser);

  // clear parser variables
  $this->aValues = array();
  $this->aTags = array();
}

/* Descr.: read pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on success, false otherwise
 * Notes: 
 * ToDo:
**/
function read($sFileName){
  // clean old data
  $this->dataParser();
  
  // create parser
  $this->oParser = xml_parser_create();

  // open file
  if(!($hFile = @fopen($sFileName, "r"))){
    $this->_error("Cannot open .data file");
    return false;
  }
  $this->sDataFile = implode("", file($sFileName));

  // parse data
  if(!@xml_parse_into_struct($this->oParser, $this->sDataFile, $this->aValues, $this->aTags)){
    $sError = "XML syntax error at line ";
    $sError .= xml_get_current_line_number($this->oParser);
    $sError .= ": " . xml_error_string(xml_get_error_code($this->oParser));
    $this->_error($sError);
    return false;
  }
  
  fclose($hFile);
  return $this->_parse();
}

/* Descr.: write/create pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
**/
function write($sFileName){
  global $sIncludePath;

  // reading data template file
  $sTemplFile = $sIncludePath . "/data.templ";
  $hFile = @fopen($sTemplFile, "r");
  if(!$hFile){
    $this->_error("Cannot open data template file");
    return false;
  }
  $sOutput = fread($hFile, filesize($sTemplFile));
  fclose($hFile);

  // reading entries template file
  $sTemplFile = $sIncludePath . "/entry.templ";
  $hFile = @fopen($sTemplFile, "r");
  if(!$hFile){
    $this->_error("Cannot open entry template file");
    return false;
  }
  $sEntryTempl = fread($hFile, filesize($sTemplFile));
  fclose($hFile);

  // reading marks template file
  $sTemplFile = $sIncludePath . "/mark.templ";
  $hFile = @fopen($sTemplFile, "r");
  if(!$hFile){
    $this->_error("Cannot open entry template file");
    return false;
  }
  $sMarkTempl = fread($hFile, filesize($sTemplFile));
  fclose($hFile);

  // building output file
  $sOutput = str_replace(":NAME:", $this->sDataName, $sOutput);

  // building entries 
  // for each entry
  while(list($sEntryId, $aMarks) = each($this->aEntries)){
    $sThisEntry = str_replace(":ID:", $sEntryId, $sEntryTempl);
    $sThisEntry = str_replace(":DATE:", $aMarks[DATE], $sThisEntry);

    // building marks
    // for each mark
    $sMarks = "";
    while(list($sMarkName, $sMarkValue) = each($aMarks)) {
      if($sMarkName == "DATE") 
        continue;

      $sThisMark = str_replace(":NAME:", $sMarkName, $sMarkTempl);
      $sMarkValue = $this->_html_to_xml($sMarkValue);
      $sThisMark = str_replace(":CONTENT:", $sMarkValue, $sThisMark);

      $sMarks .= $sThisMark;
    }
    $sThisEntry = str_replace(":MARKS:", $sMarks, $sThisEntry);

    $sEntries .= $sThisEntry . "\n";
  }

  $sOutput = str_replace(":ENTRIES:", $sEntries, $sOutput);

  // writing output file
  $hFile = @fopen($sFileName, "w");	// yes, I'll write everything again
  if(!$hFile){
    $this->_error("Cannot open output file for writing");
    return false;
  }
  fwrite($hFile, $sOutput);
  fclose($hFile);

  return true;
}

/* Descr.: convert html to xml compatible
 * Params.:
 *  $sData - data to be converted
 * Return:
 *  converted data
 * Notes:
 * ToDo:
**/
function _html_to_xml($sData) {
  $sData = str_replace("<", "&lt;", $sData);
  $sData = str_replace(">", "&gt;", $sData);

  return $sData;
}




/* Descr.: parse pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on succes, false otherwise
 * Notes:
 * ToDo:
**/
function _parse(){
  $this->ctValues = 0;

  // aElement	- tag
  //		- type (open/close/complete/cdata)
  // 		- level (use to ignore xhtml)
  // 		- attributes
  //		- 
  $aElement = $this->aValues[$this->ctValues];

  // is this a .data xml?
  if($aElement[tag] != "DATA" && $aElement[type] != "open"){
    $this->_error("DATA element expected");
    return false;
  }

  // pub name
  if(!isset($aElement[attributes][NAME])){
    $this->_error("DATA NAME attribute expected");
    return false;
  }
  $this->sDataName = $aElement[attributes][NAME];

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while(isset($this->aValues[$this->ctValues])){
    switch($aElement[tag]){
    case "ENTRY":
      if(!$this->_entry())
        return false;
      break;

   }

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }
  return true;
}

/* Descr.: entry handling
 * Params.:
 * Return:
 *   header retrieved from struct
 * Notes:
 * ToDo:
*/
function _entry(){

  $aElement = $this->aValues[$this->ctValues];

  if(!isset($aElement[attributes][ID])){
    $this->_error("ENTRY ID attribute expected");
    return false;
  }
  $this->iCurrentId = $iId = $aElement[attributes][ID];

  if(isset($this->aEntries[$iId])){
    $this->warning("Duplicated ENTRY, ignoring first occurrence");
    unset($this->aEntries[$iId]);
  }

  if(isset($aElement[attributes][DATE]))
    $this->aEntries[$iId][DATE] = $aElement[attributes][DATE];

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while($aElement[tag] != "ENTRY" || $aElement[type] != "close"){
    switch($aElement[tag]){
    case "MARK":
      if(!$this->_mark())
        return false;
      break;
    }

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }

  return true;
}

/* Descr.: mark handling
 * Params.:
 * Return:
 *   mark retrieved from struct
 * Notes:
 * ToDo:
**/
function _mark(){  
  $aElement = $this->aValues[$this->ctValues];
  $iId = $this->iCurrentId;

  if(!isset($aElement[attributes][NAME])){
    $this->_error("MARK NAME attribute expected");
    return false;
  }
  $sName = $aElement[attributes][NAME];

  if(isset($this->aEntries[$iId][$sName])){
    $this->_warning("Duplicated ENTRY MARK, ignoring first occurrence");
    unset($this->aEntries[$iId][$sName]);
  }

  if($aElement[type] == "complete"){
    $this->aEntries[$iId][$sName] = $aElement[value];
    return true;
  }
  
  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  $sContent = "";
  while($aElement[tag] != "MARK" || $aElement[type] != "close"){
    $sContent .= $aElement[value];

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }
  $this->aEntries[$iId][$sName] = $sContent;

  return true;
}

/* Descr.: retrieve error description
 * Params.:
 * Return:
 *   error description, null if none
 * Notes:
 * ToDo:
**/
function getError(){
  return $this->sError;
}

/* Descr.: set error variables
 * Params.:
 *   $sDescr - error description
 * Return:
 *   none
 * Notes:
 *   NEVER set $sDescr to null
 * ToDo:
**/
function _error($sDescr){
  $this->sError = $sDescr;
}


/* Descr.: retrieve warnings descriptions
 * Params.:
 * Return:
 *   warnings list, empty array if none
 * Notes:
 * ToDo:
**/
function getWarnings(){
  return $this->aWarnings;
}

/* Descr.: add warning
 * Params.:
 *   $sDescr - warning description
 * Return:
 *   none
 * Notes:
 * ToDo:
**/
function _warning($sDescr){
  $this->aWarnings[$this->ctWarnings] = $sDescr;
  $this->ctWarnings++;
}



}	// class pubParser

}	// if(!def...
?>
