<?php
/********************************************************************
* mini-pub - Mini PHP Publisher
* -----------------------------
*
* Copyright (C) 2002 Fabio A Mazzarino
* Released under the terms of GNU General Public License version 2
* See COPYING file for more information
*
* File: classMiniPub.inc.php
* Descr.: mini-pub main class
*
* Authors:
*   Fabio A Mazzarino <mazza at altern.org>
*
* ToDo:
*   . weekdays, multiple languages (difficult, hun?)
*   . many :DATE:XX: marks to be planed (and documented)
*
*******************************************************************/

if(!defined("__CLASS_MINI_PUB_INC_PHP__")){
define("__CLASS_MINI_PUB_INC_PHP__", "classMiniPub.inc.php");


// retrieving include dir
$aIncluded = get_included_files();
while(list($id, $sFile) = each($aIncluded)){
  $iPos = strpos($sFile, "classMiniPub.inc.php");
  if($iPos){		// this is the right include
    $sIncludePath = dirname($sFile);
    break;
  }
}

include($sIncludePath . "/classPubParser.inc.php");
include($sIncludePath . "/classDataParser.inc.php");

class miniPub {

var $aPubDataFiles;	// $aPubDataFiles[$sPubFile] = $sDataFile
var $sLayoutFile;

var $sError;


/* Descr.: default constructor
 * Parms.:
 *   $aPubFiles - pub files applicable to layout file
 *   $sLayoutFile - layout file
 * Return:
 * Notes:
 * ToDo:
 *   optimize memory allocation (?)
**/
function miniPub($sLayoutFile){
  // avoiding errors
  $this->aDataFiles = array();

  // constructor parameters
  $this->sLayoutFile = $sLayoutFile;
}

function addPubDataFiles($sPubFile, $sDataFile, $nSkip, $nShow, 
                         $sEntryId = null){
  $this->aDataFiles[$sPubFile] = $sDataFile;
  $this->aDataFileSkip[$sPubFile] = $nSkip;
  $this->aDataFileShow[$sPubFile] = $nShow;
  $this->aDataEntryId[$sPubFile] = $sEntryId;
}

function delPubDataFiles($sPubFile) {
  unset($this->aDataFiles[$sPubFile]);
  unset($this->aDataFileSkip[$sPubFile]);
  unset($this->aDataFileShow[$sPubFile]);
  unset($this->aDataEntryId[$sPubFile]);
}

function clearPubDataFiles(){
  $this->aDataFiles = array();
  $this->aDataFileSkip = array();
  $this->aDataFileShow = array();
  $this->aDataEntryId = array();
}

/* Descr.: preview page 
 * Params.:
 * Return:
 *   page preview, null on error
 * Notes:
 * ToDo:
**/
function previewPage(){
  // read Layout file content
  $hFile = @fopen($this->sLayoutFile, "r");
  if(!$hFile) {
    $this->_error("Layout Template file not found");
    return null;
  }
  $sLayout = fread($hFile, filesize($this->sLayoutFile));
  fclose($hFile);

  // parse pub files and apply to layout file
  // for each .pub and .data file combo
  while(list($sPubFile, $sDataFile) = each($this->aDataFiles)){
    $sEntries = "";

    // retrieving .pub file
    $oPub = new pubParser();
    if(!$oPub->read($sPubFile)){
      $sError = "parsing error on file " . $sPubFile . ": ";
      $sError .= $oPub->getError();
      $this->_error($sError);
      return null;
    }

    // retrieving .data file
    $oData = new dataParser();
    if(!$oData->read($sDataFile)){
      $sError = "parsing error on file " . $sDataFile . ": ";
      $sError .= $oData->getError();
      $this->_error($sError);
      return null;
    }

    // header
    $sEntries .= $oPub->sHeader;

    // filling .pub template
    // template counters
    $aTempls = $oPub->aTempls;
    $nTempls = $oPub->ctTempls;
    $ctTempl = 0;
    // entry counters
    $ctEntries = 0;
    $ctShown = 0;
    $nSkip = $this->aDataFileSkip[$sPubFile];
    $nShow = $this->aDataFileShow[$sPubFile];
    $sEntryId = $this->aDataEntryId[$sPubFile];

    // for each entry
    while(list($sId, $aData) = each($oData->aEntries)){
      // entry before of after nSkip or after nShow
      if($ctEntries < $nSkip) {
        $ctEntries++;
        continue;
      }
      if($ctShown >= $nShow && $nShow > 0)
        break;

      // save template to a draft
      $sEntry = $aTempls[$ctTempl];

      // if entry id is set then single entry mode
      if(isset($sEntryId) || $sEntryId != null) {
        $sId = $sEntryId;
	$aData = $oData->aEntries[$sId];
      }

      // for each mark replace at templ
      // special marks
      // id mark - :ID:
      $sTemplMark = ":ID:";
      $sEntry = str_replace($sTemplMark, $sId, $sEntry);
      // date marks
      // retrieve date array
      $aDate[year] = substr($aData[DATE], 0, 4);
      $aDate[month] = substr($aData[DATE], 4, 2);
      $aDate[mday] = substr($aData[DATE], 6, 2);
      // replace date marks - :DATE:XX:
      $sTemplMark = ":DATE:YY:";	// year (4 digit)
      $sEntry = str_replace($sTemplMark, $aDate[year], $sEntry);
      $sTemplMark = ":DATE:MM:";	// month
      $sEntry = str_replace($sTemplMark, $aDate[month], $sEntry);
      $sTemplMark = ":DATE:DD:";	// month day
      $sEntry = str_replace($sTemplMark, $aDate[mday], $sEntry);

      if(!is_array($aData)) $aData = array();
      while(list($sMark, $sData) = each($aData)){
        $sTemplMark = ":" . $sMark . ":";
        $sEntry = str_replace($sTemplMark, $sData, $sEntry);
      }
      // save modification
      $sEntries .= $sEntry;
      // templ loop
      $ctTempl++;
      if($ctTempl >= $nTempls)
        $ctTempl = 0;

      // entries count
      $ctEntries++;
      // shown count
      $ctShown++;

      if(isset($sEntryId) || $sEntryId != null)
        break;
    }

    // footer
    $sEntries .= $oPub->sFooter;

    // building and replacing layout mark
    $sMark = "<!-- " . $oPub->sPubName . " -->";
    $sLayout = str_replace($sMark, $sEntries, $sLayout);
  }

  return $sLayout;
}


/* Descr.: generate page
 * Params.:
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
**/
function generatePage($sOutputFile){

  $sOutput = $this->previewPage();

  $hFile = @fopen($sOutputFile, "w");	// if exists eliminates
  if(!$hFile){
    $this->_error("Cannot create output file");
    return false;
  }
  fwrite($hFile, $sOutput);
  fclose($hFile);

  return true;
}

/* Descr.: retrieve error description
 * Params.:
 * Return:
 *   error description, null if none
 * Notes:
 * ToDo:
**/
function getError(){
  return $this->sError;
}


/* Descr.: internal function to set error variables
 * Params.:
 *   $sDescr - error description
 * Return:
 * Notes:
 * ToDo:
**/
function _error($sDescr){
  $this->sError = $sDescr;
}


}	// class miniPub

}	// if(!def...
?>
