<?php
/********************************************************************
* mini-pub - Mini PHP Publisher
* -----------------------------
*
* Copyright (C) 2002 Fabio A Mazzarino
* Released under the terms of GNU General Public License version 2
* See COPYING file for more information
*
* File: classPubParser
* Descr.: xml parser for mini-pub .pub files
*
* Authors:
*   Fabio A Mazzarino <mazza at altern.org>
*
* Changes
*   May/24/2002 - mazza - Initial version
*   Jun/13/2002 - mazza - New parsing technich, using xml struct
*   Jan/15/2003 - mazza - fixing pub file reading (infinite loop)
*
* ToDo:
*
*******************************************************************/

if(!defined("__CLASS_PUB_PARSER_INC_PHP__")){
define("__CLASS_PUB_PARSER_INC_PHP__", "classPubParser.inc.php");

// retrieving include dir
$aIncluded = get_included_files();
while(list($id, $sFile) = each($aIncluded)){
  $iPos = strpos($sFile, "classPubParser.inc.php");
  if($iPos){             // this is the right include
    $sIncludePath = dirname($sFile);
    break;
  }
}


class pubParser{

// parser
var $oParser;
var $aValues;
var $ctValues;
var $aTags;

// .pub data
var $sPubName;
var $sHeader;
var $aTempls;
var $sFooter;

// error handling
var $sError;
var $iErrorLine;
// warning handling
var $aWarnings;
var $aWarningsLines;
var $ctWarnings;


/* Descr.: default constructor to miniPub class
 * Params.:
 * Return:
 *   none
 * Notes:
 * ToDo:
**/
function pubParser(){
  // setting error variables to no error
  $this->sError = null;
  $this->iErrorLine = 0;

  $this->aWarnings = array();
  $this->aWarningsLines = array();
  $this->ctWarnings = 0;

  // multiple templ support
  $this->aTempls = array();
  $this->ctTempls = 0;

  // public variables
  unset($this->sPubName);
  unset($this->sHeader);
  unset($this->sFooter);
  $this->aTempls = array();

  // parser variables
  unset($this->oParser);
  $this->aValues = array();
  $this->aTags = array();
  unset($this->ctValues);
}

/* Descr.: read pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on success, false otherwise
 * Notes: 
 * ToDo:
**/
function read($sFileName){
  // clear old data
  $this->pubParser();

  // create parser
  $this->oParser = xml_parser_create();

  // open file
  if(!($hFile = @fopen($sFileName, "r"))){
    $this->_error("Cannot open .pub file");
    return false;
  }
  $this->sPubFile = implode("", file($sFileName));

  // parse data
  if(!@xml_parse_into_struct($this->oParser, $this->sPubFile, $this->aValues, $this->aTags)){
    $sError = "XML syntax error at line ";
    $sError .= xml_get_current_line_number($this->oParser);
    $sError .= ": " . xml_error_string(xml_get_error_code($this->oParser));
    $this->_error($sError);
    return false;
  }

  fclose($hFile);
  return $this->_parse();
}

/* Descr.: write/create pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
**/
function write($sFileName){
  global $sIncludePath;

  // reading pub template file
  $sTemplFile = $sIncludePath . "/pub.templ";
  $hFile = @fopen($sTemplFile, "r");
  if(!$hFile){
    $this->_error("Cannot open template file");
    return false;
  }
  $sOutput = fread($hFile, filesize($sTemplFile));
  fclose($hFile);

  // reading templ template file
  $sTemplFile = $sIncludePath . "/templ.templ";
  $hFile = @fopen($sTemplFile, "r");
  if(!$hFile){
    $this->_error("Cannot open template file");
    return false;
  }
  $sTemplTempl = fread($hFile, filesize($sTemplFile));
  fclose($hFile);

  // preparing html code (header and footer)
  $this->sHeader = $this->_html_to_xml($this->sHeader);
  $this->sFooter = $this->_html_to_xml($this->sFooter);

  // building output file
  $sOutput = str_replace(":PUBNAME:", $this->sPubName, $sOutput);
  
  $sOutput = str_replace(":HEAD:" , $this->sHeader, $sOutput);
  $sOutput = str_replace(":FOOT:", $this->sFooter, $sOutput);

  // building multiple templates
  while(list($sId, $sTempl) = each($this->aTempls)){
    $sTempl = $this->_html_to_xml($sTempl);
    $sThisTempl = str_replace(":TEMPL:", $sTempl, $sTemplTempl);
    $sTempls .= $sThisTempl;
  }
  $sOutput = str_replace(":TEMPLS:", $sTempls, $sOutput);

  // writing output file
  $hFile = @fopen($sFileName, "w");	// yes, I'll write everything again
  if(!$hFile){
    $this->_error("Cannot open output file for writing");
    return false;
  }
  fwrite($hFile, $sOutput);
  fclose($hFile);

  return true;
}

/* Descr.: convert html to xml compatible
 * Params.:
 *  $sData - data to be converted
 * Return:
 *  converted data
 * Notes:
 * ToDo:
 **/
function _html_to_xml($sData) {
  $sData = str_replace("<", "&lt;", $sData);
  $sData = str_replace(">", "&gt;", $sData);

  return $sData;
}





/* Descr.: parse pub file
 * Params.:
 *   $sFileName - pub file
 * Return:
 *   true on succes, false otherwise
 * Notes:
 * ToDo:
**/
function _parse(){
  $this->ctValues = 0;

  // aElement	- tag
  //		- type (open/close/complete/cdata)
  // 		- level (use to ignore xhtml)
  // 		- attributes
  //		- 
  $aElement = $this->aValues[$this->ctValues];

  // is this a .pub xml?
  if($aElement[tag] != "PUB" && $aElement[type] != "open"){
    $this->_error("PUB element expected");
    return false;
  }

  // pub name
  if(!isset($aElement[attributes][NAME])){
    $this->_error("PUB NAME attribute expected");
    return false;
  }
  $this->sPubName = $aElement[attributes][NAME];

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while(isset($this->aValues[$this->ctValues])){
    switch($aElement[tag]){
    case "HEAD":
      if(!$this->_header())
        return false;
      break;
  
    case "TEMPL":
      if(!$this->_templ())
        return false;
      break;
  
    case "FOOT":
      if(!$this->_footer())
        return false;
      break;
    }

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }
  return true;
}

/* Descr.: header handling
 * Params.:
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
*/
function _header(){
  if(isset($this->sHeader))
    $this->_warning("duplicated HEAD element, ignoring first occurrence");

  $this->sHeader = "";
  $aElement = $this->aValues[$this->ctValues];

  if($aElement[type] == "complete"){
    $this->sHeader .= $aElement[value];
    $this->ctValues++;
    return true;
  }

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while($aElement[tag] != "HEAD" || $aElement[type] != "close"){
    $this->sHeader .= $aElement[value];

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }

  return true;
}

/* Descr.: templates handling
 * Params.:
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
**/
function _templ(){  

  $aElement = $this->aValues[$this->ctValues];

  if($aElement[type] == "complete"){
    $this->ctValues++;
    $this->aTempls[$this->ctTempls] = $aElement[value];
    $this->ctTempls++;
    return true;
  }

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while($aElement[tag] != "TEMPL" || $aElement[type] != "close"){
    $sTempl .= $aElement[value];

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }

  $this->aTempls[$this->ctTempls] = $sTempl;
  $this->ctTempls++;

  return true;
}

/* Descr.: footer handling
 * Params.:
 * Return:
 *   true on success, false otherwise
 * Notes:
 * ToDo:
**/
function _footer(){
  if(isset($this->sFooter))
    $this->_warning("duplicated FOOT element, ignoring first occurrence");

  $this->sFooter = "";
  $aElement = $this->aValues[$this->ctValues];

  if($aElement[type] == "complete"){
    $this->sFooter = $aElement[value];
    return true;
  }

  $this->ctValues++;
  $aElement = $this->aValues[$this->ctValues];

  while($aElement[tag] != "FOOT" || $aElement[type] != "close"){
    $this->sFooter .= $aElement[value];

    $this->ctValues++;
    $aElement = $this->aValues[$this->ctValues];
  }
  return true;
}

/* Descr.: retrieve error description
 * Params.:
 * Return:
 *   error description, null if none
 * Notes:
 * ToDo:
**/
function getError(){
  return $this->sError;
}

/* Descr.: set error variables
 * Params.:
 *   $sDescr - error description
 * Return:
 *   none
 * Notes:
 *   NEVER set $sDescr to null
 * ToDo:
**/
function _error($sDescr){
  $this->sError = $sDescr;
}


/* Descr.: retrieve warnings descriptions
 * Params.:
 * Return:
 *   warnings list, empty array if none
 * Notes:
 * ToDo:
**/
function getWarnings(){
  return $this->aWarnings;
}

/* Descr.: add warning
 * Params.:
 *   $sDescr - warning description
 * Return:
 *   none
 * Notes:
 * ToDo:
**/
function _warning($sDescr){
  $this->aWarnings[$this->ctWarnings] = $sDescr;
  $this->ctWarnings++;
}






}	// class pubParser

}	// if(!def...
?>
